clear, close all
demo_number=1;
verbose='on';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Hyperspectral image definitions
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch demo_number 
    
  case 1
       
       noise = 'off';
       SNR=0;                  % signal-to-noise ratio in dB
       ro_n=0;         		    % correlation coef.

       ilum_fluct = 'off';
       q=10;                   % parameter to control ilummination fluctuations
       signat_var = 'off';
       c_v=0;                  % parameter to control signature variability

       if strcmp (verbose, 'on'), fprintf(1,'***\t\t DEMO %d > random independent abundances (uniform distributed)\n',demo_number);end
       
  case 2
       
       noise = 'off';
       SNR=0;                  % signal-to-noise ratio in dB
       ro_n=0;        		    % correlation coef.

       ilum_fluct = 'off';
       q=10;                   % parameter to control ilummination fluctuations
       signat_var = 'off';
       c_v=0;                  % parameter to control signature variability
       
       if strcmp (verbose, 'on'), fprintf(1,'***\t\t DEMO %d > random dependent abundances (dirichlet distributed)\n',demo_number);end
       
  case 3
       
       noise = 'off';
       SNR=0;                  % signal-to-noise ratio in dB
       ro_n=0;        		    % correlation coef.

       ilum_fluct = 'off';
       q=10;                   % parameter to control ilummination fluctuations
       signat_var = 'off';
       c_v=0;                  % parameter to control signature variability

       if strcmp (verbose, 'on'), fprintf(1,'***\t\t DEMO %d > abundances on strips (dependent)\n',demo_number);end
       
  case 4
       
       noise = 'on';
       SNR=30;                 % signal-to-noise ratio in dB
       ro_n=0;        		    % correlation coef.

       ilum_fluct = 'on';
       q=10;                   % parameter to control ilummination fluctuations
       signat_var = 'on';
       c_v=0.8;                % parameter to control signature variability

       if strcmp (verbose, 'on'), fprintf(1,'***\t\t DEMO %d > abundances on strips (dependent)\n',demo_number);end       
       
   case 5

       load cuprite_ref
       %[Lines Columns L] = size(x);
       N = Lines * Columns; % number of pixels
       p = 6;
       A = zeros(L,p);
       s = zeros(p,N);
       x_n = x;clear x;

       if strcmp (verbose, 'on'), fprintf(1,'***\t\t DEMO %d > real data Cuprite (AVIRIS- reflectance data)\n',demo_number);end       
       if strcmp (verbose, 'on'), fprintf(1,'***\t\t searching for %d independent components\n',p);end       
       
otherwise         
       error('unknown demo number');
end        



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Generate sintetic hyperspectral image
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch demo_number 
    
  case 1
         p=3;		%  number of endmembers

         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         % random independent abundances with uniform distributions 
         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         Lines = 36;
         Columns = 36;
         N = Lines * Columns; % number of pixels
         s_o = rand(N,p);

  case 2
         p=3;		%  number of endmembers

         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         % abundances (dependent) with Dirichlet distributions 
         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         Lines = 36;
         Columns = 36;
         N = Lines * Columns; % number of pixels
         s_o = dirichlet_rnd([ones(1,p)/p],N);

  case { 3 , 4 }
         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         % abundances on strips
         % 3 endmembers (p=3) mixed in the image 
         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         a = [1.0 0.5 0.2 0.0 0.2 0.0;
              0.0 0.3 0.7 1.0 0.5 0.0;
              0.0 0.2 0.1 0.0 0.3 1.0];
         [p nt]=size(a);
         Lines = 36;
         Columns = 36;
         N = Lines * Columns; % number of pixels
         dim_t = Columns/nt;
         s_o = reshape(repmat( reshape(a',[1 p*nt]),[dim_t*Lines 1]),[N p]);
end %switch
      
if demo_number ~= 5     
         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         % signatures from USGS Library 
         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         load signatures
         if strcmp (verbose, 'on'),
            %fprintf('Available signatures:\n%s',names');
            fprintf('***\t\tSelected signatures:\n%s',names(1:p,:)');
         end
         [L p_max]=size(A);
         A = A(:,1:p);
         BANDS=1:L;

         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         % illumination fluctuation
         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         if strcmp (ilum_fluct, 'on'),
            s_v = s_o .* random('beta',q,1,N,p);
         else
            s_v = s_o;
         end
         
         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         % signature variability
         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         if strcmp (signat_var, 'on'),
            variab = 1+c_v*(-0.5+rand(1,N*p)) ; % between   [1-cv/2 1+cv/2]
            %variab = 1-(1-c_v)*rand(1,N*p) ;    % between   [cv 1]

            s_r = reshape(repmat(reshape(s_v,[p*N 1]),[1 L])',[L N p]);
            A_r = reshape(repmat(A',[1 N])',[L N p]);
            variab_r = reshape(repmat(variab,[L 1]),[L N p]);

            x = sum(A_r .* s_r .* variab_r ,3);
         else
            x = A * s_v' ;               
         end
            
         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         % adding noise 
         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         if strcmp (noise, 'on'),
            varianc = sum(x(:).^2)/10^(SNR/10) /L/N ;
            n = sqrt(varianc)*randn([L N]);
         else
            n = 0;
         end

         x_n = x + n;
         
         
     
        a=nan*ones(224,p);
        a(BANDS,:)= A;     
 
        if strcmp (verbose, 'on'),
           SNR=10*log10(sum(x(:).^2)/sum(n(:).^2));
           fprintf(1,'***\t\t SNR= %gdB\n',SNR);
           fprintf(1,'***\t\t Ilumination perturbation: %s\n', ilum_fluct ) 
           fprintf(1,'***\t\t Signature variability: %s \n', signat_var ) 
        end

        figure(2);
        S=reshape(s_v,[Lines Columns p]);
        for i=1:p,
            subplot(p,4,4*i-2),
                 imagesc(S(:,:,i));%colormap('gray');
                 if i==1, title('True abundance');end
            subplot(p,4,4*i-3),
                 plot(wavlen,a(:,i),'r');%axis([.3 2.6 -.2 1.2]);
        end
        
end %if demo_number ~=5
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %
        % Unmixing Process
        %
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        [s_est, A_est, W]=fastica(x_n,'lastEig', p, 'numOfIC', p,...
                                      'verbose','off','displayMode','off',...
                                      'maxNumIterations',200);
        s_est=s_est';
        
        
if demo_number ~= 5     
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %
        % Collecting Results
        %
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        [dummy, N_ic]=size(A_est);        
        if isempty(A_est)
           A_est=zeros(L,p);
           s_est=zeros(N,p);
        elseif N_ic~=p 
           d = p-N_ic; % number of components not detected
           A_est(:,N_ic+1:p)=zeros(L,d);
           s_est(:,N_ic+1:p)=zeros(N,d);
           if strcmp (verbose, 'on'),
              fprintf('***\t\t Warning: FastICA estimate only %g independent components\n', N_ic ) 
           end   
        end
        
        eval(['warning off MATLAB:divideByZero'])
        CRD = corrcoef([A A_est]);
        D = CRD(p+1:p+N_ic,1:p);   
        % permute results
        perm_mtx = zeros(p,p);     % it should be perm_mtx=zeros(N_ic,p);
                                   % there is a line with zeros if N_ic~=p!
        for i=1:N_ic 
              [ld cd]=find(max(abs(D(:)))==abs(D)); 
              ld=ld(1);cd=cd(1); % in the case os more than one maximum
              perm_mtx(ld,cd) = D(ld,cd)/abs(D(ld,cd)); % can be +/-1;
              D(:,cd)=0*ones(N_ic,1); % one signal is represented by one ic
              D(ld,:)=0*ones(1,p); % one ic only represents one signal
        end
        A_est = A_est * perm_mtx;
        s_est = s_est * perm_mtx;

        coef_A = sqrt(diag(A_est'*A_est)./diag(A'*A));	            
        A_est  = A_est ./ repmat(coef_A',[L 1]);
        coef_s = sqrt(diag(s_est'*s_est)./diag(s_v'*s_v));
        s_est  = s_est ./ repmat(coef_s',[N 1]);
        
        
        a_est=nan*ones(224,p);
        a_est(BANDS,:)= A_est;     

        S_est = reshape(s_est,[Lines Columns p]); 
        figure(2)
        for i=1:p
            subplot(p,4,4*i-3),
                 hold on;plot(wavlen,a_est(:,i),'b');hold off;%axis([.3 2.6 -.2 1.2]);
                 if i==1, title('signatures true and estimate');end
            subplot(p,4,4*i-1),
                 imagesc(S_est(:,:,i));%axis([300 2500 -.2 1.2]);
                 if i==1, title('abundance estimate');end
            subplot(p,4,4*i),
                 plot(1:Columns,S_est(1,:,i),'m',1:Columns,S(1,:,i),'b');
                 if i==1, title('line of abundance true and estimate');end
        end

else %if demo_number ~= 5     
          %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          % show components
          %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     
          S_est = reshape(s_est,[Lines Columns p]);
          a_est=nan*ones(224,p);
          a_est(BANDS,:)= A_est;     
          
          n=ceil(sqrt(p));
          for i= 1:p
              figure(1)
                subplot(n,n,i);
                imagesc(S_est(:,:,i));
                colormap gray
                axis('off','equal')
                eval(['title(''component ' int2str(i) ''')']);
              figure(2)  
                subplot(n,n,i);
                plot(wavlen,a_est(:,i),'b-','Linewidth',2);axis([400 2500 min(a_est(:,i)) max(a_est(:,i))]);
                eval(['title(''component ' int2str(i) ''')']);
                xlabel('wavelength (\mum)'); ylabel('reflectance (%)');
          end
end %if demo_number ~= 5     
   
          


